#ifndef _LINUX_DAC_H
#define _LINUX_DAC_H

/*
 * include file for data acquisition driver
 * David Schleef
 */

#include <linux/time.h>
#include <linux/timex.h>

#define DAC_MAJOR  60
#define DAC_DEVICES 16

#define DAC_CONFIG 1
#define DAC_GETINFO 2
#define DAC_ADTRIG 3
#define DAC_DATRIG 4
#define DAC_ADMODE 5
#define DAC_COPY_TO 6
#define DAC_COPY_FROM 7
#define DAC_MEMSET 8
#define DAC_EXEC 9
#define DAC_RESET 10
#define DAC_CHANINFO 11
#define DAC_CHANCFG 12
#define DAC_STATUS 13
#define DAC_GET 14
#define DAC_PUT 15

#define DAC_ANALOG 0x1
#define DAC_DIGITAL 0
#define DAC_INPUT 0x2
#define DAC_OUTPUT 0

#define dac_isinput(a) ((a).flags&DAC_INPUT)
#define dac_isoutput(a) (!((a).flags&DAC_INPUT))
#define dac_isanalog(a) ((a).flags&DAC_ANALOG)
#define dac_isdigital(a) (!((a).flags&DAC_ANALOG))

typedef struct{
	int chan;
	int data;
	struct timeval time;
}dac_s;

typedef struct{
	int dest;
	int src;
	int nbytes;
}dac_copy;

typedef struct dac_device_struct dac_device;

typedef struct{
	int flags;
	int bits;
	dac_s last;
	double zero;
	double delta;
	int (*trig)(dac_device *,dac_s *);
}dac_chinfo;

typedef struct{
	char name[20];
	int iobase;
	int irq;
}dac_devconfig;

#ifdef __KERNEL__

typedef int dac_devinit(dac_device *,dac_devconfig *);

typedef struct dac_bufll{
	struct dac_bufll *next;
	int use_count;
	char stuff[0];
}dac_bufll;

typedef struct{
	dac_bufll *first;
	dac_bufll *last;
	int in;
	int use_count;
	int num_pages;
}dac_buf;

typedef struct{
	dac_buf *buf;
	dac_bufll *bufll;
	int offset;
}dac_bufptr;

struct dac_device_struct{
	int use_count;
	int minor;
	int iobase;
	int iosize;
	int irq;
	int chan;
	dac_chinfo *chinfo;
	dac_buf rdbuf;
	struct wait_queue *rdwait;
	void *it;
	int (*rem)(dac_device *);
	int (*memcpy_to)(dac_device *,int dest,void *src,int nbytes);
	int (*memcpy_from)(dac_device *,void *dest,int src,int nbytes);
	int (*d_memset)(dac_device *,int dest,int nbytes,int val);
	int (*memexec)(dac_device *,int entry_pc);
	int (*reset)(dac_device *,int entry_pc);
	int (*write)(dac_device *,char *src,int nbytes);
};

extern dac_device *dac_devices;
extern dac_devinit *dac_devinits[];
extern const int dac_maxinits;

/*
 * function prototypes
 */

int dac_initbuf(dac_buf *buf);
void dac_freebuf(dac_buf *buf);
void copy_to_buffer(dac_buf *buf,void *src,int n);
int copy_from_buffer(dac_bufptr *ptr,void *dest,int n);
dac_bufptr *dac_hookbuf(dac_buf *buf);
void dac_unhookbuf(dac_bufptr *it);

#endif

#endif /* _LINUX_DAC_H */

