#include <linux/malloc.h>
#include <linux/string.h>
#include <asm/errno.h>
#include <linux/kdev_t.h>
#include <dac.h>

static void dec_use(dac_buf *buf,dac_bufll *it);

int dac_initbuf(dac_buf *buf)
{
	buf->use_count=0;
	buf->in=0;
	buf->first=(dac_bufll *)get_free_page(GFP_KERNEL);
	buf->last=buf->first;
	buf->num_pages=1;
	buf->first->next=NULL;
	buf->first->use_count=1;  /* 1 for the interrupt routine */

	return 0;
}

void dac_freebuf(dac_buf *buf)
{
	/* there *should* only be one page to free. */
	if(buf->num_pages>1){
		printk("dac: dac_freebuf called with %d pages in buffer\n",buf->num_pages);
	}
	if(buf->last->use_count>1){
		printk("dac: dac_freebuf called with last having use_count of %d\n",
			buf->last->use_count);
	}
	dec_use(buf,buf->last);
	buf->last=NULL;
}

void copy_to_buffer(dac_buf *buf,void *src,int n)
{
	int nbytes;
	dac_bufll *old_last;

	if(!buf->use_count)return;
	while(n>0){
		nbytes=n;
		if(nbytes>=PAGE_SIZE-sizeof(dac_bufll)-buf->in){
			nbytes=PAGE_SIZE-sizeof(dac_bufll)-buf->in;
			memcpy(buf->last->stuff+buf->in,src,nbytes);

			if(!(buf->last->next=
				(dac_bufll *)get_free_page(GFP_ATOMIC)))
				return;
			buf->num_pages++;
			old_last=buf->last;
			buf->last=old_last->next;
			buf->last->use_count=buf->use_count+1;
/*printk("alloc bufll 0x%08x\n",buf->last);*/
			dec_use(buf,old_last);
			buf->in=0;
		}else{
			memcpy(buf->last->stuff+buf->in,src,nbytes);
			buf->in+=nbytes;
		}
		src+=nbytes;
		n-=nbytes;
	}
}

int copy_from_buffer(dac_bufptr *ptr,void *dest,int n)
{
	int nbytes,i,count=0;
	dac_bufll *old_bufll;
	long flags;

	while(n>0){
		nbytes=n;
		save_flags(flags);
		cli();
		if(ptr->bufll->next==NULL){
			i=ptr->buf->in-ptr->offset;
		}else{
			i=PAGE_SIZE-sizeof(dac_bufll)-ptr->offset;
		}
		restore_flags(flags);
		if(nbytes>i)nbytes=i;
		memcpy(dest+count,ptr->bufll->stuff+ptr->offset,nbytes);
		n-=nbytes;
		count+=nbytes;
		ptr->offset+=nbytes;
		if(ptr->offset>=PAGE_SIZE-sizeof(dac_bufll)){
			old_bufll=ptr->bufll;
			ptr->bufll=ptr->bufll->next;
			ptr->offset=0;
			dec_use(ptr->buf,old_bufll);
		}else{
			break;
		}
	}
	return count;
}

static void dec_use(dac_buf *buf,dac_bufll *it)
{
	it->use_count--;
/*printk("dec_use of 0x%08x\n",it);*/
	if(!it->use_count){
		if(buf->first!=it){
			printk("dac: freeing bufll that isn't first\n");
		}
		buf->first=it->next;
		buf->num_pages--;
		free_page((long)it);
	}
}

dac_bufptr *dac_hookbuf(dac_buf *buf)
{
	dac_bufptr *it;

	it=(dac_bufptr *)kmalloc(sizeof(dac_bufptr),GFP_KERNEL);
	it->buf=buf;
	buf->use_count++;

	it->bufll=buf->last;  /* these 3 lines should be atomic */
	it->offset=buf->in;
	it->bufll->use_count++;
/*printk("inc_use of 0x%08x\n",it->bufll);*/
/*printk("it->bufll->use_count after hookup=%d\n",it->bufll->use_count);*/

	return it;
}

void dac_unhookbuf(dac_bufptr *it)
{
	dac_bufll *next,*tmp;

	it->offset=0;
	next=it->bufll;
	while(next){
		tmp=next->next;
		dec_use(it->buf,next);
		next=tmp;
	}
	it->buf->use_count--;
	kfree(it);
}


