#include <linux/module.h>

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fcntl.h>
#include <linux/timer.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <asm/io.h>
#include <dac.h>

dac_device *dac_devices;

static int dac_ioctl(struct inode * inode,struct file * file,unsigned int cmd,unsigned long arg)
{
	int retval,i;
	kdev_t minor=MINOR(inode->i_rdev);
	dac_device *dev=dac_devices+minor;

	switch(cmd){
	case DAC_CONFIG:
/* used for overall configuation */
	{
		dac_devconfig db;

		if(!suser())
			return -EPERM;
#if OLD
		retval=verify_area(VERIFY_READ,(void *)arg,
			sizeof(dac_devconfig));
		if(retval)
			return retval;
		memcpy_fromfs(&db,(void *)arg,sizeof(dac_devconfig));
#else
		memcpy(&db,(void *)arg,sizeof(dac_devconfig));
#endif
		if(dev->rem){
			retval=dev->rem(dev);
			if(retval<0)return retval;
		}
		memset(dev,0,sizeof(dac_device));
		dev->minor=minor;
		if(!db.name[0])return 0;
		for(i=0;i<dac_maxinits;i++){
			retval=dac_devinits[i](dev,&db);
			if(retval<0)
				return retval;
			if(retval)return 0;
		}
		return -EIO;
	}
	case DAC_GETINFO:
/* returns overall info */
#if OLD
		retval=verify_area(VERIFY_WRITE,(void *)arg,sizeof(dac_device));
		if(retval)
			return retval;
		memcpy_tofs((void *)arg,dev,sizeof(dac_device));
#else
		memcpy((void *)arg,dev,sizeof(dac_device));
#endif
		return 0;
	case DAC_CHANINFO:
/* returns info (range, number of bits, input, output, gain) for a
particular channel */
/* FIXME */
	{
		int chan;

#if OLD
		retval=verify_area(VERIFY_READ,(void *)arg,sizeof(int));
		if(retval)return retval;
		memcpy_fromfs(&chan,(void *)arg,sizeof(int));
#else
		memcpy(&chan,(void *)arg,sizeof(int));
#endif
		/* note: flags in struct gets loaded into chan */
		if(chan>dev->chan)return -EIO;
#if OLD
		retval=verify_area(VERIFY_WRITE,(void *)arg,sizeof(dac_chinfo));
		if(retval)return retval;
		memcpy_tofs((void *)arg,dev->chinfo+chan,sizeof(dac_chinfo));
#else
		memcpy((void *)arg,dev->chinfo+chan,sizeof(dac_chinfo));
#endif
		
		return 0;
	}
	case DAC_CHANCFG:
/* changes software-configured settings for individual channels (range,
input, output, gain, etc.) */
		return 0;
	case DAC_STATUS:
/* returns last sample for each channel */
/* FIXME */
		return 0;
	case DAC_GET:
/* triggers input */
	{
		dac_s it;

#if OLD
		retval=verify_area(VERIFY_READ,(void *)arg,sizeof(dac_s));
		if(retval)
			return retval;
		memcpy_fromfs(&it,(void *)arg,sizeof(dac_s));
#else
		memcpy(&it,(void *)arg,sizeof(dac_s));
#endif
		if(it.chan>dev->chan)return -EIO;
		if(!dac_isinput(dev->chinfo[it.chan]))return -EIO;
		do_gettimeofday(&it.time);
		dev->chinfo[it.chan].trig(dev,&it);
		dev->chinfo[it.chan].last=it;
#if OLD
		retval=verify_area(VERIFY_WRITE,(void *)arg,sizeof(dac_s));
		if(retval)return retval;
		memcpy_tofs((void *)arg,&it,sizeof(dac_s));
#else
		memcpy((void *)arg,&it,sizeof(dac_s));
#endif
		
		return 0;
	}
	case DAC_PUT:
/* triggers output */
	{
		dac_s it;

#if OLD
		retval=verify_area(VERIFY_READ,(void *)arg,sizeof(dac_s));
		if(retval)return retval;
		memcpy_fromfs(&it,(void *)arg,sizeof(dac_s));
#else
		memcpy(&it,(void *)arg,sizeof(dac_s));
#endif
		if(it.chan>dev->chan)return -EIO;
		if(!dac_isoutput(dev->chinfo[it.chan]))return -EIO;
		do_gettimeofday(&it.time);
		dev->chinfo[it.chan].trig(dev,&it);
		dev->chinfo[it.chan].last=it;
#if OLD
		retval=verify_area(VERIFY_WRITE,(void *)arg,sizeof(dac_s));
		if(retval)return retval;
		memcpy_tofs((void *)arg,&it,sizeof(dac_s));
#else
		memcpy((void *)arg,&it,sizeof(dac_s));
#endif
		
		return 0;
	}
#if 0
	case DAC_DATRIG:
	{
		dac_datrig datrig;

		if(!dev->datrig)
			return -EIO;
		retval=verify_area(VERIFY_READ,(void *)arg,sizeof(dac_datrig));
		if(retval)
			return retval;
		memcpy_fromfs(&datrig,(void *)arg,sizeof(dac_datrig));
		
		retval=dev->datrig(dev,&datrig);
		if(retval)
			return retval;

		retval=verify_area(VERIFY_WRITE,(void *)arg,sizeof(dac_datrig));
		if(retval)
			return retval;
		memcpy_tofs((void *)arg,&datrig,sizeof(dac_datrig));

		return 0;
	}
	case DAC_ADMODE:
		if(!dev->admode)
			return -EIO;
		return dev->admode(dev,arg);
#endif 0
#ifdef DSP
	case DAC_COPY_TO:
	{
		dac_copy it;
		void *ptr;

		if(!dev->memcpy_to)
			return -EIO;
		retval=verify_area(VERIFY_READ,(void *)arg,sizeof(dac_copy));
		if(retval)
			return retval;
		memcpy_fromfs(&it,(void *)arg,sizeof(dac_copy));
		retval=verify_area(VERIFY_READ,(void *)it.src,it.nbytes);
		if(retval)
			return retval;
		ptr=kmalloc(it.nbytes,GFP_KERNEL);
		if(!ptr)
			return -ENOMEM;
		memcpy_fromfs(ptr,(void *)it.src,it.nbytes);

		retval=dev->memcpy_to(dev,it.dest,ptr,it.nbytes);
		kfree(ptr);
		return retval;
	}
	case DAC_COPY_FROM:
	{
		dac_copy it;
		void *ptr;

		if(!dev->memcpy_from)
			return -EIO;
		retval=verify_area(VERIFY_READ,(void *)arg,sizeof(dac_copy));
		if(retval)
			return retval;
		memcpy_fromfs(&it,(void *)arg,sizeof(dac_copy));
		retval=verify_area(VERIFY_WRITE,(void *)it.src,it.nbytes);
		if(retval)
			return retval;
		ptr=kmalloc(it.nbytes,GFP_KERNEL);
		if(!ptr)
			return -ENOMEM;
		retval=dev->memcpy_from(dev,ptr,it.src,it.nbytes);
		if(!retval)
			memcpy_tofs((void *)it.dest,ptr,it.nbytes);
		kfree(ptr);
		return retval;
	}
	case DAC_MEMSET:
	{
		dac_copy it;

		if(!dev->d_memset)
			return -EIO;
		retval=verify_area(VERIFY_READ,(void *)arg,sizeof(dac_copy));
		if(retval)
			return retval;
		memcpy_fromfs(&it,(void *)arg,sizeof(dac_copy));
		return dev->d_memset(dev,it.dest,it.nbytes,it.src);
	}
	case DAC_EXEC:
		if(!dev->memexec)
			return -EIO;
		return dev->memexec(dev,arg);
	case DAC_RESET:
		if(!dev->reset)
			return -EIO;
		return dev->reset(dev,arg);
#endif
	default:
		return -EIO;
	}
}

static long dac_write(struct inode *inode,struct file *file,const char *buf,unsigned long nbytes)
{
	kdev_t minor=MINOR(inode->i_rdev);
	dac_device *dev=dac_devices+minor;
	/*struct wait_queue wait={current,NULL};*/
	int retval;

	if(!dev->write)return -EIO;
	if(!nbytes)return 0;

#if OLD
	retval=verify_area(VERIFY_READ,(void *)buf,nbytes);
	if(retval)
		return retval;
#endif
	return dev->write(dev,buf,nbytes);
}

static long dac_read(struct inode *inode,struct file * file,char *buf,unsigned long nbytes)
{
	kdev_t minor=MINOR(inode->i_rdev);
	dac_device *dev=dac_devices+minor;
	int n,count=0,retval=0;
	struct wait_queue wait={current,NULL};

	if(!file->private_data)return -EIO;
	if(!nbytes)return 0;

	add_wait_queue(&dev->rdwait,&wait);
	current->state=TASK_INTERRUPTIBLE;
	while(nbytes>0){
		n=copy_from_buffer((dac_bufptr *)(file->private_data),buf+count,nbytes);
		if(!n){
			if(file->f_flags&O_NONBLOCK){
				retval=-EAGAIN;
				break;
			}
			if(current->signal & ~current->blocked){
				retval=-ERESTARTSYS;
				break;
			}
			schedule();
			continue;
		}
		count+=n;
		nbytes-=n;
	}
	current->state=TASK_RUNNING;
	remove_wait_queue(&dev->rdwait,&wait);

	return (count ? count : retval);
}

/*
static int dac_select(struct inode * inode,struct file * file,int sel_type,select_table *wait)
{
	return 0;
}
*/

static long long dac_lseek(struct inode *inode,struct file *file,off_t offset,int origin)
{
	return -ESPIPE;
}

static int dac_open(struct inode *inode,struct file *file)
{
	kdev_t minor=MINOR(inode->i_rdev);
	dac_device *dev=dac_devices+minor;

	if(minor>=DAC_DEVICES)return -ENODEV;
	if(!dac_devices[minor].rem && !suser())
		return -ENODEV;
	MOD_INC_USE_COUNT;
	dac_devices[minor].use_count++;
	
	if(dev->rdbuf.last)
		file->private_data=(void *)dac_hookbuf(&(dev->rdbuf));
	else file->private_data=NULL;

	return 0;
}

static void dac_close(struct inode *inode,struct file *file)
{
	kdev_t minor=MINOR(inode->i_rdev);

	if(file->private_data)
		dac_unhookbuf((dac_bufptr *)(file->private_data));
	MOD_DEC_USE_COUNT;
	dac_devices[minor].use_count--;
}

static struct file_operations dac_fops={
	dac_lseek,
	dac_read,
	dac_write,
	NULL, /* dac_readdir */
	NULL, /* dac_poll, */
	dac_ioctl,
	NULL, /* dac_mmap */
	dac_open,
	dac_close,
	NULL, /* fsync */
	NULL, /* fasync */
	NULL, /* check media change */
	NULL, /* revalidate */
};

int init_module(void)
{
	if(register_chrdev(DAC_MAJOR,"dac",&dac_fops)){
		printk("dac: unable to get major %d\n",DAC_MAJOR);
		return -EIO;
	}
	dac_devices=(dac_device *)kmalloc(sizeof(dac_device)*DAC_DEVICES,GFP_KERNEL);
	if(!dac_devices)
		return -ENOMEM;
	memset(dac_devices,0,sizeof(dac_device)*DAC_DEVICES);
	
	return 0;
}

void cleanup_module(void)
{
	int i;

	if(MOD_IN_USE)
		printk("dac: module in use -- remove delayed\n");
	for(i=0;i<DAC_DEVICES;i++){
		if(dac_devices[i].rem)
			dac_devices[i].rem(dac_devices+i);
	}

	unregister_chrdev(DAC_MAJOR,"dac");
}

