#ifndef _LINUX_DT2811_H
#define _LINUX_DT2811_H

/*
 * header file for Data Translation DT2811 driver
 * David Schleef
 */

/*

0x00    ADCSR R/W  A/D Control/Status Register
  bit 7 - (R) 1 indicates A/D conversion done
          reading ADDAT clears bit
          (W) ignored
  bit 6 - (R) 1 indicates A/D error
          (W) ignored
  bit 5 - (R) 1 indicates A/D busy, cleared at end
          of conversion
          (W) ignored
  bit 4 - (R) 0
          (W) 
  bit 3 - (R) 0
  bit 2 - (R/W) 1 indicates interrupts enabled
  bits 1,0 - (R/W) mode bits
     00  single conversion on ADGCR load
     01  continuous conversion, internal clock,
         (clock enabled on ADGCR load)
     10  continuous conversion, internal clock,
         external trigger
     11  continuous conversion, external clock,
         external trigger

0x01    ADGCR R/W A/D Gain/Channel Register
  bit 6,7 - (R/W) gain select
    00  gain=1, both PGH, PGL models
    01  gain=2 PGH, 10 PGL
    10  gain=4 PGH, 100 PGL
    11  gain=8 PGH, 500 PGL
  bit 4,5 - reserved
  bit 3-0 - (R/W) channel select
    channel number from 0-15

0x02,0x03 (R) ADDAT A/D Data Register
          (W) DADAT0 D/A Data Register 0
  0x02 low byte
  0x03 high byte

0x04,0x05 (W) DADAT0 D/A Data Register 1

0x06 (R) DIO0 Digital Input Port 0
     (W) DIO1 Digital Output Port 1

0x07 TMRCTR (R/W) Timer/Counter Register
  bits 6,7 - reserved
  bits 5-3 - Timer frequency control (mantissa)
    543  divisor  freqency (kHz)
    000  1        600
    001  10       60
    010  2        300
    011  3        200
    100  4        150
    101  5        120
    110  6        100
    111  12       50
  bits 2-0 - Timer frequency control (exponent)
    543  multiply divisor/divide frequency by
    000  1
    001  10
    010  100
    011  1000
    100  10000
    101  100000
    110  1000000
    111  10000000

*/

/*
 * flags
 */

/* ADCSR */

#define DT2811_ADDONE   0x80
#define DT2811_ADERROR  0x40
#define DT2811_ADBUSY   0x20
#define DT2811_CLRERROR 0x10
#define DT2811_INTENB   0x04
#define DT2811_ADMODE   0x03


#define DT2811_ADCSR (dt2811.base+0)
#define DT2811_ADGCR (dt2811.base+1)
#define DT2811_ADDATLO (dt2811.base+2)
#define DT2811_ADDATHI (dt2811.base+3)
#define DT2811_DADAT0LO (dt2811.base+2)
#define DT2811_DADAT0HI (dt2811.base+3)
#define DT2811_DADAT1LO (dt2811.base+4)
#define DT2811_DADAT1HI (dt2811.base+5)
#define DT2811_DIO (dt2811.base+6)
#define DT2811_TMRCTR (dt2811.base+7)

/*
 * function prototypes
 */

extern int dt2811_init(void);

#endif _LINUX_DT2811_H

